package net.knarcraft.ffmpegconverter.converter.sorter;

import net.knarcraft.ffmpegconverter.streams.SubtitleStream;
import net.knarcraft.ffmpegconverter.utility.OutputUtil;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

/**
 * A sorter for filtering subtitle streams by title
 */
public class SubtitleTitleSorter extends AbstractSorter<SubtitleStream> {

    private final List<String> titleFilter;

    /**
     * Instantiates a new subtitle title sorter
     *
     * <p>If a simple string, or invalid RegEx is given, any stream containing the titleFilter in its title will be
     * retained. If a valid RegEx is given, any stream matching the titleFilter is retained.</p>
     *
     * @param titleFilter <p>The filter to use. RegEx match, or a string the title must contain.</p>
     */
    public SubtitleTitleSorter(@NotNull List<String> titleFilter) {
        this.titleFilter = new ArrayList<>(titleFilter);
    }

    @Override
    public @NotNull List<SubtitleStream> sort(@NotNull List<SubtitleStream> input) {
        // Don't change anything if no filter is given
        this.titleFilter.removeIf(String::isBlank);
        if (this.titleFilter.isEmpty()) {
            return input;
        }

        List<SubtitleStream> output = new ArrayList<>();

        for (String filter : this.titleFilter) {
            if (filter.trim().isEmpty()) {
                continue;
            }

            boolean isRegEx = isValidRegularExpression(filter) && hasSpecialRegexCharacters(filter);

            OutputUtil.printDebug("Filtering subtitles by filter " + filter + ". RegEx is " + isRegEx);

            for (SubtitleStream subtitleStream : input) {
                String title = subtitleStream.getTitle().trim().toLowerCase();
                // Add the subtitle if the filter matches, and it hasn't been added already
                boolean matches = filter.trim().equals("*") || (isRegEx && title.matches(filter.toLowerCase())) ||
                        (!isRegEx && title.contains(filter.toLowerCase()));
                if (matches && !output.contains(subtitleStream)) {
                    OutputUtil.printDebug("Subtitle stream with title " + title + " matches");
                    output.add(subtitleStream);
                }
            }
        }
        return output;
    }

    /**
     * Checks whether the given string is a valid regular expression
     *
     * @param input <p>The string to check</p>
     * @return <p>True if the given string has no invalid expressions</p>
     */
    private static boolean isValidRegularExpression(@NotNull String input) {
        try {
            Pattern.compile(input);
            return true;
        } catch (PatternSyntaxException e) {
            return false;
        }
    }

    /**
     * Checks whether the input string has any RegEx special characters
     *
     * @param input <p>The input to check</p>
     * @return <p>True if RegEx characters exist in the string</p>
     */
    private static boolean hasSpecialRegexCharacters(String input) {
        Pattern regexSpecialCharacters = Pattern.compile("[\\\\.\\[\\]{}()<>*+\\-=!?^$|]");
        return regexSpecialCharacters.matcher(input).find();
    }

}
