package net.knarcraft.ffmpegconverter.streams;

import net.knarcraft.ffmpegconverter.utility.ValueParsingHelper;
import org.jetbrains.annotations.NotNull;

import java.util.Map;

/**
 * An abstract implementation of a stream object implementing common methods
 */
public abstract class AbstractStream implements StreamObject {

    protected final int inputIndex;
    protected final int absoluteIndex;
    protected final int relativeIndex;
    protected final String codecName;
    protected final String language;
    protected final boolean isDefault;
    protected final String title;
    protected int outputIndex;

    /**
     * Instantiates a new abstract stream
     *
     * @param streamInfo    <p>All info about the stream</p>
     * @param inputIndex    <p>The index of the input file this stream belongs to</p>
     * @param relativeIndex <p>The relative index of this stream, only considering streams of the same type</p>
     */
    protected AbstractStream(@NotNull Map<StreamTag, String> streamInfo, int inputIndex, int relativeIndex) {
        this.codecName = ValueParsingHelper.parseString(streamInfo.get(StreamTag.CODEC_NAME), "");
        this.absoluteIndex = ValueParsingHelper.parseInt(streamInfo.get(StreamTag.INDEX), -1);
        this.language = parseLanguage(streamInfo);
        this.isDefault = ValueParsingHelper.parseBoolean(streamInfo.get(StreamTag.DISPOSITION_DEFAULT), false);
        this.title = ValueParsingHelper.parseString(streamInfo.get(StreamTag.TAG_TITLE), "");
        this.inputIndex = inputIndex;
        this.relativeIndex = relativeIndex;
        this.outputIndex = -1;
    }

    @Override
    @NotNull
    public String getCodecName() {
        return this.codecName;
    }

    @Override
    public int getAbsoluteIndex() {
        return this.absoluteIndex;
    }

    @Override
    public int getRelativeIndex() {
        return this.relativeIndex;
    }

    @Override
    @NotNull
    public String getLanguage() {
        return this.language;
    }

    @Override
    public boolean isDefault() {
        return this.isDefault;
    }

    @Override
    @NotNull
    public String getTitle() {
        return this.title;
    }

    @Override
    public int getInputIndex() {
        return this.inputIndex;
    }

    @Override
    public int getOutputIndex() {
        return this.outputIndex;
    }

    @Override
    public void setOutputIndex(int newIndex) {
        this.outputIndex = newIndex;
    }

    /**
     * Parses the correct language of a stream
     *
     * <p>As some people tend to set weird incorrect languages for anime streams, this tries to correct that by
     * detecting streams that are actually in english, but set to something else.</p>
     *
     * @param streamInfo <p>All info about the stream</p>
     * @return <p>The actual language</p>
     */
    @NotNull
    private String parseLanguage(@NotNull Map<StreamTag, String> streamInfo) {
        String languageString = ValueParsingHelper.parseString(streamInfo.get(StreamTag.TAG_LANGUAGE), "und");
        String title = ValueParsingHelper.parseString(streamInfo.get(StreamTag.TAG_TITLE), "");
        if (languageString.equalsIgnoreCase("zxx") || languageString.equalsIgnoreCase("enm") ||
                (title.toLowerCase().matches(".*english.*") && languageString.equalsIgnoreCase("jpn"))) {
            return "eng";
        }
        if (languageString.equalsIgnoreCase("jap")) {
            return "jpn";
        }
        return languageString;
    }

    @Override
    @NotNull
    public String toString() {
        return this.title + " | Input index: " + inputIndex + " | Language: " + language;
    }

}